//
//  DBAPI2WaitForChangesInDBFolderTask.m
//
//

#ifdef DROPBOX_FEATURE

#import "DBAPI2WaitForChangesInDBFolderTask.h"

#import "KPNumberRange.h"

#import "KPLogger.h"

#if !__has_feature(objc_arc)
#error This class requires automatic reference counting
#endif

static NSTimeInterval const MAX_TIMEOUT = (5*60);

@interface DBAPI2WaitForChangesInDBFolderTask ()

// Input:
@property (nonatomic, nonnull, copy) NSString *cursor;
@property (nonatomic, assign) NSTimeInterval timeout;
@property (nonatomic, assign) BOOL processOffline;

// Output:
@property (nonatomic, assign) BOOL changes;
@property (nullable, nonatomic, strong) DBFILESListFolderLongpollError *routeError;

@end

@implementation DBAPI2WaitForChangesInDBFolderTask

- (nonnull instancetype)initWithCursor:(nonnull NSString *)cursor
                               timeout:(NSTimeInterval)timeout {
    
    if (self = [super init]) {
        
        _cursor = cursor;
        
        _timeout =
            [NUMBER_RANGE_OPENED_CLOSED(0.0, MAX_TIMEOUT) doubleValueIfInRange:timeout
                                                               orFallbackValue:MAX_TIMEOUT];
    }
    
    return self;
}

- (nullable DBTask *)doInternalTaskAsynchronously {
    
    DBRpcTask<DBFILESListFolderLongpollResult *, DBFILESListFolderLongpollError *> *dbTask =
        [[self fileAPI] listFolderLongpoll:_cursor
                                   timeout:@(_timeout)];
    
    weakify(self);
    [dbTask setResponseBlock:^(DBFILESListFolderLongpollResult * __nullable result,
                               DBFILESListFolderLongpollError * __nullable routeError,
                               DBRequestError * __nullable requestError) {
        
        strongify(self);
        
        self.changes = [result.changes boolValue];
        self.routeError = routeError;
        [self updateRequestError:requestError];
        
        [self stopWaiting];
        
    }];
    
    return dbTask;
}

- (BOOL)shouldShowActivityIndicator {
    
    // BY DEFAULT IS YES
    
    return NO;
}


- (void)cleanupOutput {
    
    [super cleanupOutput];
    
    _changes = NO;
    _routeError = nil;
}

- (void)logWorkResults {
    
    KPLog(@"\n[self: %@]\n[changes: %d]\n[routeError: %@]\n[requestError: %@]",
          self, _changes, _routeError, self.requestError);
}

- (BOOL)returnResult {
    
    return (_routeError == nil) && (self.requestError == nil);
}

@end

#endif
